package com.example.application.views.helloworld;

import com.example.application.views.MainLayout;
import com.vaadin.flow.component.AttachEvent;
import com.vaadin.flow.component.DetachEvent;
import com.vaadin.flow.component.UI;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.router.PageTitle;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.router.RouteAlias;
import org.springframework.context.ApplicationContext;

@PageTitle("Hello World")
@Route(value = "hello", layout = MainLayout.class)
@RouteAlias(value = "", layout = MainLayout.class)
public class HelloWorldView extends VerticalLayout {

    private final ApplicationContext ctx;

    public HelloWorldView(ApplicationContext ctx) {
        this.ctx = ctx;

        UI ui = UI.getCurrent();
        add("To reproduce: ");
        add("Click button 1 few times to ensure that you get same instance of UISCoped bean. ");
        add("Click button 2 and then observe by clicking button 1 that now the instance is different => not expected behavior. ");
        add("Also observe that same instance of UI and Route received detach and attach. => not expected behavior. ");
        add(new Button("1. Print instance of UIScopeBean to console",
                e -> {
                    System.out.println("UIScopeBean instance is " + ctx.getBean(UIScopeBean.class).hashCode());
                }));
        add(new Button("2. Break @UIScope by simulating proxy error", e -> {
            new Thread() {
                @Override
                public void run() {
                    try {
                        Thread.sleep(100);
                    } catch (InterruptedException ex) {
                        throw new RuntimeException(ex);
                    }
                    ui.access(() -> {
                        //Without a proxy this is very hard to reproduce, but internally ServerRpcHandler calls these
                        //when proxy error occurs.
                        ui.getInternals().getStateTree().prepareForResync();
                        ui.getInternals().getDependencyList().clearPendingSendToClient();
                    });
                }
            }.start();

        }));

        UI.getCurrent().addAttachListener(ev -> {
            System.out.println("UI got attached. Instance: " + ev.getUI().hashCode());
        });
        UI.getCurrent().addDetachListener(ev -> {
            System.out.println("UI got detached. Instance: " + ev.getUI().hashCode());
        });
    }

    @Override
    protected void onAttach(AttachEvent attachEvent) {
        super.onAttach(attachEvent);
        System.out.println("Route got attached. Instance: " + this.hashCode());
    }

    @Override
    protected void onDetach(DetachEvent detachEvent) {
        super.onDetach(detachEvent);
        System.out.println("Route got detached. Instance: " + this.hashCode());
    }
}
